@@ -22,8 +22,8 @@ RW_MAX_SAMPLES_PER_EPISODE = TMRL_CONFIG["RW_MAX_SAMPLES_PER_EPISODE"]  # If thi
 
 PRAGMA_RNN = False  # True to use an RNN, False to use an MLP
 
-PRAGMA_CUDA_TRAINING = TMRL_CONFIG["CUDA_TRAINING"]  # True if CUDA, False if CPU (trainer)
-PRAGMA_CUDA_INFERENCE = TMRL_CONFIG["CUDA_INFERENCE"]  # True if CUDA, False if CPU (rollout worker)
+CUDA_TRAINING = TMRL_CONFIG["CUDA_TRAINING"]  # True if CUDA, False if CPU (trainer)
+CUDA_INFERENCE = TMRL_CONFIG["CUDA_INFERENCE"]  # True if CUDA, False if CPU (rollout worker)
 
 PRAGMA_GAMEPAD = TMRL_CONFIG["VIRTUAL_GAMEPAD"]  # True to use gamepad, False to use keyboard
 
@@ -50,6 +50,9 @@ IMG_HIST_LEN = ENV_CONFIG["IMG_HIST_LEN"]  # 4 without RNN, 1 with RNN
 ACT_BUF_LEN = ENV_CONFIG["RTGYM_CONFIG"]["act_buf_len"]
 WINDOW_WIDTH = ENV_CONFIG["WINDOW_WIDTH"]
 WINDOW_HEIGHT = ENV_CONFIG["WINDOW_HEIGHT"]
+GRAYSCALE = ENV_CONFIG["IMG_GRAYSCALE"] if "IMG_GRAYSCALE" in ENV_CONFIG else False
+IMG_WIDTH = ENV_CONFIG["IMG_WIDTH"] if "IMG_WIDTH" in ENV_CONFIG else 64
+IMG_HEIGHT = ENV_CONFIG["IMG_HEIGHT"] if "IMG_HEIGHT" in ENV_CONFIG else 64
 
 # DEBUGGING AND BENCHMARKING: ===================================
 
@@ -57,6 +60,7 @@ CRC_DEBUG = False  # Only for checking the consistency of the custom networking
 CRC_DEBUG_SAMPLES = 100  # Number of samples collected in CRC_DEBUG mode
 PROFILE_TRAINER = False  # Will profile each epoch in the Trainer when True
 SYNCHRONIZE_CUDA = False  # Set to True for profiling, False otherwise
+DEBUG_MODEL = TMRL_CONFIG["DEBUG_MODEL_EVERY"] if "DEBUG_MODEL_EVERY" in TMRL_CONFIG.keys() else -1
 
 # FILE SYSTEM: =================================================
 
@@ -65,10 +69,10 @@ logging.debug(f" PATH_DATA:{PATH_DATA}")
 
 MODEL_HISTORY = TMRL_CONFIG["SAVE_MODEL_EVERY"]  # 0 for not saving history, x for saving model history every x new model received by RolloutWorker
 
-MODEL_PATH_WORKER = str(WEIGHTS_FOLDER / (RUN_NAME + ".pth"))
+MODEL_PATH_WORKER = str(WEIGHTS_FOLDER / (RUN_NAME + ".tmod"))
 MODEL_PATH_SAVE_HISTORY = str(WEIGHTS_FOLDER / (RUN_NAME + "_"))
-MODEL_PATH_TRAINER = str(WEIGHTS_FOLDER / (RUN_NAME + "_t.pth"))
-CHECKPOINT_PATH = str(CHECKPOINTS_FOLDER / RUN_NAME)
+MODEL_PATH_TRAINER = str(WEIGHTS_FOLDER / (RUN_NAME + "_t.tmod"))
+CHECKPOINT_PATH = str(CHECKPOINTS_FOLDER / (RUN_NAME + "_t.tcpt"))
 DATASET_PATH = str(DATASET_FOLDER)
 REWARD_PATH = str(REWARD_FOLDER / "reward.pkl")
 
@@ -85,22 +89,15 @@ os.environ['WANDB_API_KEY'] = WANDB_KEY
 
 PRINT_BYTESIZES = True
 
-PORT_TRAINER = TMRL_CONFIG["PORT_TRAINER"]  # Port to listen on (non-privileged ports are > 1023)
-PORT_ROLLOUT = TMRL_CONFIG["PORT_ROLLOUT"]  # Port to listen on (non-privileged ports are > 1023)
+PORT = TMRL_CONFIG["PORT"]  # Port to listen to (non-privileged ports are > 1023)
+LOCAL_PORT_SERVER = TMRL_CONFIG["LOCAL_PORT_SERVER"]
+LOCAL_PORT_TRAINER = TMRL_CONFIG["LOCAL_PORT_TRAINER"]
+LOCAL_PORT_WORKER = TMRL_CONFIG["LOCAL_PORT_WORKER"]
+PASSWORD = TMRL_CONFIG["PASSWORD"]
+SECURITY = "TLS" if TMRL_CONFIG["TLS"] else None
+CREDENTIALS_DIRECTORY = TMRL_CONFIG["TLS_CREDENTIALS_DIRECTORY"] if TMRL_CONFIG["TLS_CREDENTIALS_DIRECTORY"] != "" else None
+HOSTNAME = TMRL_CONFIG["TLS_HOSTNAME"]
+NB_WORKERS = None if TMRL_CONFIG["NB_WORKERS"] < 0 else TMRL_CONFIG["NB_WORKERS"]
+
 BUFFER_SIZE = TMRL_CONFIG["BUFFER_SIZE"]  # 268435456  # socket buffer size (200 000 000 is large enough for 1000 images right now)
 HEADER_SIZE = TMRL_CONFIG["HEADER_SIZE"]  # fixed number of characters used to describe the data length
-
-SOCKET_TIMEOUT_CONNECT_TRAINER = TMRL_CONFIG["SOCKET_TIMEOUT_CONNECT_TRAINER"]
-SOCKET_TIMEOUT_ACCEPT_TRAINER = TMRL_CONFIG["SOCKET_TIMEOUT_ACCEPT_TRAINER"]
-SOCKET_TIMEOUT_CONNECT_ROLLOUT = TMRL_CONFIG["SOCKET_TIMEOUT_CONNECT_ROLLOUT"]
-SOCKET_TIMEOUT_ACCEPT_ROLLOUT = TMRL_CONFIG["SOCKET_TIMEOUT_ACCEPT_ROLLOUT"]  # socket waiting for rollout workers closed and restarted at this interval
-SOCKET_TIMEOUT_COMMUNICATE = TMRL_CONFIG["SOCKET_TIMEOUT_COMMUNICATE"]
-SELECT_TIMEOUT_OUTBOUND = TMRL_CONFIG["SELECT_TIMEOUT_OUTBOUND"]
-ACK_TIMEOUT_WORKER_TO_SERVER = TMRL_CONFIG["ACK_TIMEOUT_WORKER_TO_SERVER"]
-ACK_TIMEOUT_TRAINER_TO_SERVER = TMRL_CONFIG["ACK_TIMEOUT_TRAINER_TO_SERVER"]
-ACK_TIMEOUT_SERVER_TO_WORKER = TMRL_CONFIG["ACK_TIMEOUT_SERVER_TO_WORKER"]
-ACK_TIMEOUT_SERVER_TO_TRAINER = TMRL_CONFIG["ACK_TIMEOUT_SERVER_TO_TRAINER"]
-RECV_TIMEOUT_TRAINER_FROM_SERVER = TMRL_CONFIG["RECV_TIMEOUT_TRAINER_FROM_SERVER"]
-RECV_TIMEOUT_WORKER_FROM_SERVER = TMRL_CONFIG["RECV_TIMEOUT_WORKER_FROM_SERVER"]
-WAIT_BEFORE_RECONNECTION = TMRL_CONFIG["WAIT_BEFORE_RECONNECTION"]
-LOOP_SLEEP_TIME = TMRL_CONFIG["LOOP_SLEEP_TIME"]
