@@ -9,11 +9,11 @@ import cv2
 
 def obs_preprocessor_tm_act_in_obs(obs):
     """
-    Preprocessor for TM2020 with images, converting images back to uint8
+    Preprocessor for TM2020 full environment with grayscale images
     """
     grayscale_images = obs[3]
     grayscale_images = grayscale_images.astype(np.float32) / 256.0
-    obs = (obs[0], obs[1], obs[2], grayscale_images, *obs[4:])  # >= 1 action
+    obs = (obs[0] / 1000.0, obs[1] / 10.0, obs[2] / 10000.0, grayscale_images, *obs[4:])  # >= 1 action
     return obs
 
 
