@@ -17,9 +17,8 @@ def save_replays(nb_replays=np.inf):
     rw = RolloutWorker(env_cls=partial(GenericGymEnv, id="real-time-gym-v0", gym_kwargs={"config": config}),
                        actor_module_cls=partial(cfg_obj.POLICY),
                        sample_compressor=cfg_obj.SAMPLE_COMPRESSOR,
-                       device='cuda' if cfg.PRAGMA_CUDA_INFERENCE else 'cpu',
+                       device='cuda' if cfg.CUDA_INFERENCE else 'cpu',
                        server_ip=cfg.SERVER_IP_FOR_WORKER,
-                       min_samples_per_worker_packet=1000 if not cfg.CRC_DEBUG else cfg.CRC_DEBUG_SAMPLES,
                        model_path=cfg.MODEL_PATH_WORKER,
                        obs_preprocessor=cfg_obj.OBS_PREPROCESSOR,
                        crc_debug=cfg.CRC_DEBUG,
