@@ -15,7 +15,7 @@ class TrainingAgent(ABC):
         Args:
             observation_space (Gym.spaces.Space): observation space (here for your convenience)
             action_space (Gym.spaces.Space): action space (here for your convenience)
-            device (str): torch device that should be used for training (e.g., `"cpu"` or `"cuda:0"`)
+            device (str): device that should be used for training
         """
         self.observation_space = observation_space
         self.action_space = action_space
@@ -27,10 +27,10 @@ class TrainingAgent(ABC):
         Executes a training step.
 
         Args:
-            batch: tuple or batched torch.tensors (previous observation, action, reward, new observation, terminated, truncated)
+            batch: tuple or batched tensors (previous observation, action, reward, new observation, terminated, truncated)
 
         Returns:
-            ret_dict: dictionary: a dictionary containing one entry per metric you wish to log (e.g. for wandb)
+            dict: a dictionary containing one entry per metric you wish to log (e.g. for wandb)
         """
         raise NotImplementedError
 
@@ -40,7 +40,7 @@ class TrainingAgent(ABC):
         Returns the current ActorModule to be broadcast to the RolloutWorkers.
 
         Returns:
-             actor: ActorModule: current actor to be broadcast
+             ActorModule: current actor to be broadcast
         """
         raise NotImplementedError
 
